
#include "ELExcelHeader.h"

_ApplicationPtr pApplication ;
BOOL gbInitialized = FALSE ;

//////////////////////////////////////////////////////////////
// DLLMain
BOOL APIENTRY DllMain
( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved )
{
    switch ( ul_reason_for_call )
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
		break;
    }
    return TRUE ;
}


//////////////////////////////////////////////////////////////
// ELXL_OnDestroy
int __stdcall ELXL_OnDestroy
( IEasyLanguageObject * pEL )
{
	try {
		CStatus * pStatus ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		if( pStatus ) {
			delete pStatus ;
			pStatus = 0 ;
			pEL->Properties->Items[ "pStatus" ] = (int) 0 ;
			return 0 ;
		}
	}
	catch(...) {
		return 1 ;
		fnGenRuntimeError( pEL, ERROR_PERFORMING_CLEANUP ) ;
	}
	return 0 ;
}


/*/////////////////////////////////////////////////////////////
ELXL_Init:  Excel initialization function

Check for a running instance of Excel...
If no instance running, then attempt to start an instance...
If Excel is running or new instance is successfully started,
load pApplication with a value...
Otherwise, raise a run-time error in TradeStation.
/*/////////////////////////////////////////////////////////////
BOOL __stdcall ELXL_Init
( IEasyLanguageObject * pEL )
{
	if( !gbInitialized ) {

		try {
			gbInitialized = TRUE ;
			CLSID clsid;
			CLSIDFromProgID( L"Excel.Application", &clsid ) ;  
			IUnknown * pUnk ;
	    
			if ( !FAILED( GetActiveObject( clsid, NULL, (IUnknown**) &pUnk ) ) ) {
				IDispatch * pDisp;
				if ( !FAILED( pUnk->QueryInterface( IID_IDispatch, (void **) &pDisp ) ) ) { 
					pApplication = pDisp ;
					pApplication->PutVisible( LOCALE_USER_DEFAULT, (_variant_t) TRUE ) ;
					pUnk->Release();
					return TRUE ;
				}
			}
		}
		catch( ... ) {
			return FALSE ;
			fnGenRuntimeError( pEL, NOT_RUNNING_AND_ERROR_STARTING ) ;
		}
		/*
		else if ( !FAILED( pApplication.CreateInstance( L"Excel.Application" ) ) ) {
			return TRUE ;
		}
		*/
	}
	return TRUE ;
}


//////////////////////////////////////////////////////////////
// ELXL_UsingWorksheet
BOOL __stdcall ELXL_UsingWorksheet
( IEasyLanguageObject* pEL, LPSTR szWorkbookName, LPSTR szWorksheetName )
{
	try {

		CStatus * pStatus ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		if( pStatus )
		{
			delete pStatus ;
			pStatus = 0 ;
			pEL->Properties->Items[ "pStatus" ] = (int) 0 ;
		}
		
		_WorksheetPtr pWksheet ;
		pWksheet = fnGetWksheetPtr( szWorkbookName, szWorksheetName ) ;
		if( pWksheet ) {
			CStatus * pStatus = new CStatus( pWksheet ) ;
			pEL->Properties->Items[ "pStatus" ] = (int) pStatus ;
			return TRUE ;
		} 
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
			return FALSE ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_USING_WKSHEET ) ;			
		return FALSE ;
	}
}


//////////////////////////////////////////////////////////////
// ELXL_UsingWorksheetNum
BOOL __stdcall ELXL_UsingWorksheetNum
( IEasyLanguageObject* pEL, LPSTR szWorkbookName, int nWorksheetNum )
{
	try {

		CStatus * pStatus ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		if( pStatus )
		{
			delete pStatus ;
			pStatus = 0 ;
			pEL->Properties->Items[ "pStatus" ] = (int) 0 ;
		}
		
		_WorksheetPtr pWksheet ;
		pWksheet = fnGetWksheetNumPtr( szWorkbookName, nWorksheetNum ) ;
		if( pWksheet ) {
			CStatus * pStatus = new CStatus( pWksheet ) ;
			pEL->Properties->Items[ "pStatus" ] = (int) pStatus ;
			return TRUE ;
		} 
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
			return FALSE ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_USING_WKSHEET ) ;			
		return FALSE ;
	}
}

//////////////////////////////////////////////////////////////
// ELXL_ClearCellContents
BOOL __stdcall ELXL_ClearCellContents
( IEasyLanguageObject* pEL, LPSTR szCell )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				((RangePtr)pWksheet->GetRange( szCell, vtMissing ))->ClearContents() ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_CLEARING_CELL_CONTENTS ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
// ELXL_ClearCellContentsRC
BOOL __stdcall ELXL_ClearCellContentsRC
( IEasyLanguageObject* pEL, int nRow, int nColumn )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				((RangePtr) pWksheet->Cells->Item[ nRow, nColumn ])->ClearContents() ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_CLEARING_CELL_CONTENTS ) ;
	}
	return FALSE ;
}

//////////////////////////////////////////////////////////////
// ELXL_ClearCellFormat
BOOL __stdcall ELXL_ClearCellFormat
( IEasyLanguageObject* pEL, LPSTR szCell )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				((RangePtr)pWksheet->GetRange( szCell, vtMissing ))->ClearFormats() ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_CLEARING_CELL_FORMAT ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
// ELXL_ClearCellFormatRC
BOOL __stdcall ELXL_ClearCellFormatRC
( IEasyLanguageObject* pEL, int nRow, int nColumn )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				((RangePtr) pWksheet->Cells->Item[ nRow, nColumn ])->ClearFormats() ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_CLEARING_CELL_FORMAT ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
// ELXL_ClearSheetContents
BOOL __stdcall ELXL_ClearSheetContents
( IEasyLanguageObject* pEL )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				((RangePtr)(pWksheet->Cells))->ClearContents() ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_CLEARING_SHEET_CONTENTS ) ;
	}
	return FALSE ;
}

//////////////////////////////////////////////////////////////
// ELXL_ClearSheetFormat
BOOL __stdcall ELXL_ClearSheetFormat
( IEasyLanguageObject* pEL )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				((RangePtr)(pWksheet->Cells))->ClearFormats() ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_CLEARING_SHEET_FORMAT ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
// ELXL_SetCellValueString
BOOL __stdcall ELXL_SetCellValueString
( IEasyLanguageObject* pEL, LPSTR szCell, LPSTR szValue )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				pWksheet->GetRange( szCell, vtMissing )->PutValue( vtMissing, szValue ) ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_CELL_TO_STRING ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
// ELXL_SetCellValueStringRC
BOOL __stdcall ELXL_SetCellValueStringRC
( IEasyLanguageObject* pEL, int nRow, int nColumn, LPSTR szValue )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				pWksheet->Cells->Item[ nRow, nColumn ] = szValue ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_CELL_TO_STRING ) ;
	}
	return FALSE ;
}

//////////////////////////////////////////////////////////////
//  ELXL_SetCellValueNumber
BOOL __stdcall ELXL_SetCellValueNumber
( IEasyLanguageObject* pEL, LPSTR szCell, double dValue )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				pWksheet->GetRange( szCell, vtMissing )->PutValue( vtMissing, dValue ) ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_CELL_TO_NUMBER ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
//  ELXL_SetCellValueNumberRC
BOOL __stdcall ELXL_SetCellValueNumberRC
( IEasyLanguageObject* pEL, int nRow, int nColumn, double dValue )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				pWksheet->Cells->Item[ nRow, nColumn ] = dValue ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_CELL_TO_NUMBER ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
//  ELXL_SetCellValueBoolean
BOOL __stdcall ELXL_SetCellValueBoolean
( IEasyLanguageObject* pEL, LPSTR szCell, BOOL bValue )
{
	try {
		char szFormula[9] ;
		if ( bValue ){
			strcpy_s( szFormula, "=TRUE()" ) ;
		} 
		else {
			strcpy_s ( szFormula, "=FALSE()" ) ;
		}

		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				pWksheet->GetRange( szCell, vtMissing )->FormulaR1C1 =
					szFormula ; 
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_CELL_TO_NUMBER ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
//  ELXL_SetCellValueBooleanRC
BOOL __stdcall ELXL_SetCellValueBooleanRC
( IEasyLanguageObject* pEL, int nRow, int nColumn, BOOL bValue )
{
	try {
		char szFormula[9] ;
		if ( bValue ){
			strcpy_s( szFormula, "=TRUE()" ) ;
		} 
		else {
			strcpy_s ( szFormula, "=FALSE()" ) ;
		}

		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				((RangePtr) pWksheet->Cells->Item[ nRow, nColumn ])->FormulaR1C1 =
					szFormula ; 
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_CELL_TO_NUMBER ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
//  ELXL_SetCellBGColor
BOOL __stdcall ELXL_SetCellBGColor
( IEasyLanguageObject* pEL, LPSTR szCell, int nColorNum )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				pWksheet->GetRange( szCell, vtMissing )->Interior->Color = nColorNum ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_CELL_BG_COLOR ) ;
	}
	return FALSE ;
}

//////////////////////////////////////////////////////////////
//  ELXL_SetCellBGColorRC
BOOL __stdcall ELXL_SetCellBGColorRC
( IEasyLanguageObject* pEL, int nRow, int nColumn, int nColorNum )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				((RangePtr)pWksheet->Cells->Item[ nRow, nColumn ])->Interior->Color =
					nColorNum ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_CELL_BG_COLOR ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
//  ELXL_SetCellFontColor
BOOL __stdcall ELXL_SetCellFontColor
( IEasyLanguageObject* pEL, LPSTR szCell, int nColorNum )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				pWksheet->GetRange( szCell, vtMissing )->Font->Color = nColorNum ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_CELL_FONT_COLOR ) ;
	}
	return FALSE ;
}

//////////////////////////////////////////////////////////////
//  ELXL_SetCellFontColorRC
BOOL __stdcall ELXL_SetCellFontColorRC
( IEasyLanguageObject* pEL, int nRow, int nColumn, int nColorNum )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				((RangePtr)pWksheet->Cells->Item[ nRow, nColumn ])->Font->Color =
					nColorNum ;
				return TRUE ;
			}
			catch(...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_SETTING_CELL_FONT_COLOR ) ;
	}
	return FALSE ;
}



//////////////////////////////////////////////////////////////
//  Get Functions
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//  ELXL_GetCellValueString
BOOL __stdcall ELXL_GetCellValueString
( IEasyLanguageObject * pEL, LPSTR szCell, LPSTR szStringVarName )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				IEasyLanguageVariable * pELVar ;
				pELVar = pEL->Variables[ szStringVarName ] ;
				try {
					pELVar->AsString[0] =
					 pWksheet->GetRange( szCell, vtMissing )->GetValue2().bstrVal ;
					return TRUE ;
					// ((IEasyLanguageVariable *) pEL->Variables[ szStringVarName ])->AsString[0] =
					//  pWksheet->GetRange( szCell, vtMissing )->GetValue2().bstrVal ;
				}
				catch( ... ) {
					return FALSE ;
				}
			} 
			catch( ... ) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_GETTING_STRING_FROM_CELL ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
//  ELXL_GetCellValueStringRC
BOOL __stdcall ELXL_GetCellValueStringRC
( IEasyLanguageObject * pEL, int nRow, int nColumn, LPSTR szStringVarName )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				IEasyLanguageVariable * pELVar ;
				pELVar = pEL->Variables[ szStringVarName ] ;
				try {
					pELVar->AsString[0] =
					 ((RangePtr)pWksheet->Cells->Item[ nRow, nColumn ])->
					  GetValue2().bstrVal ;
					return TRUE ;
					// ((IEasyLanguageVariable *) pEL->Variables[ szStringVarName ])->AsString[0] =
					// ((RangePtr)pWksheet->Cells->Item[ nRow, nColumn ])->GetValue2().bstrVal ;
				}
				catch( ... ) {
					return FALSE ;
				}
			}
			catch( ... ) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_GETTING_STRING_FROM_CELL ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
//  ELXL_GetCellValueNumber
BOOL __stdcall ELXL_GetCellValueNumber
( IEasyLanguageObject * pEL, LPSTR szCell, LPSTR szNumericVarName )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				IEasyLanguageVariable * pELVar ;
				pELVar = pEL->Variables[ szNumericVarName ] ;
				try {
					pELVar->AsDouble[0] = (double) pWksheet->
					 GetRange( szCell, vtMissing )->GetValue2() ;
					return TRUE ;
					//((IEasyLanguageVariable *) pEL->Variables[ szNumericVarName ])->AsDouble[0] =
					//(double) pWksheet->GetRange( szCell, vtMissing )->GetValue2() ;
				}
				catch( ...) {
					return FALSE ;
				}
			}
			catch( ... ) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_GETTING_NUMBER_FROM_CELL ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
//  ELXL_GetCellValueNumberRC
BOOL __stdcall ELXL_GetCellValueNumberRC
( IEasyLanguageObject * pEL, int nRow, int nColumn, LPSTR szNumericVarName )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				IEasyLanguageVariable * pELVar ;
				pELVar = pEL->Variables[ szNumericVarName ] ;
				try {
					pELVar->AsDouble[0] = (double) (((RangePtr)pWksheet->
					 Cells->Item[ nRow, nColumn ])->GetValue2()) ;
					return TRUE ;
					// ((IEasyLanguageVariable *) pEL->Variables[ szNumericVarName ])->AsDouble[0] =
					// (double) ((RangePtr)pWksheet->Cells->Item[ nRow, nColumn ])->GetValue2() ;
				}
				catch( ... ) {
					return FALSE ;
				}
			}
			catch( ... ) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_GETTING_NUMBER_FROM_CELL ) ;
	}
	return FALSE ;
}

//////////////////////////////////////////////////////////////
//  ELXL_GetCellValueBoolean
BOOL __stdcall ELXL_GetCellValueBoolean
( IEasyLanguageObject * pEL, LPSTR szCell, LPSTR szBooleanVarName )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				IEasyLanguageVariable * pELVar ;
				pELVar = pEL->Variables[ szBooleanVarName ] ;
				try {
					pELVar->AsTrueFalse[0] = (BOOL) pWksheet->
					 GetRange( szCell, vtMissing )->GetValue2() ;
					return TRUE ;
					// ((IEasyLanguageVariable *) pEL->Variables[ szBooleanVarName ])->AsTrueFalse[0] =
					// (BOOL) pWksheet->GetRange( szCell, vtMissing )->GetValue2() ;
				}
				catch( ... ) {
					return FALSE ;
				}
			}
			catch( ... ) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
			fnGenRuntimeError( pEL, ERROR_GETTING_BOOLEAN_FROM_CELL ) ;
	}
	return FALSE ;
}

//////////////////////////////////////////////////////////////
//  ELXL_GetCellValueBooleanRC
BOOL __stdcall ELXL_GetCellValueBooleanRC
( IEasyLanguageObject * pEL, int nRow, int nColumn, LPSTR szBooleanVarName )
{
	try {
		CStatus * pStatus ;
		_WorksheetPtr pWksheet ;
		pStatus = ( CStatus * ) ( (int) pEL->Properties->Items[ "pStatus" ] ) ;
		pWksheet = pStatus->GetSheetPtr() ;
		if( pWksheet ) {
			try {
				IEasyLanguageVariable * pELVar ;
				pELVar = pEL->Variables[ szBooleanVarName ] ;
				try {
					pELVar->AsTrueFalse[0] = (BOOL) ((RangePtr)pWksheet->Cells->
					 Item[ nRow, nColumn ])->GetValue2() ;
					return TRUE ;
					// ((IEasyLanguageVariable *) pEL->Variables[ szBooleanVarName ])->AsTrueFalse[0] =
					//  (BOOL) ((RangePtr)pWksheet->Cells->Item[ nRow, nColumn ])->GetValue2() ;
				}
				catch ( ... ) {
					return FALSE ;
				}
			}
			catch (...) {
				return FALSE ;
			}
		}
		else {
			fnGenRuntimeError( pEL, INVALID_WORKSHEET_PTR ) ;
		}
	}
	catch(...) {
		fnGenRuntimeError( pEL, ERROR_GETTING_BOOLEAN_FROM_CELL ) ;
	}
	return FALSE ;
}


//////////////////////////////////////////////////////////////
//  Internal use functions - cannot be called from outside
//  the DLL.
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
//  fnGetWksheetPtr
_WorksheetPtr fnGetWksheetPtr
( LPSTR sWorkbookName, LPSTR sWorksheetName ) 
{
	WorkbooksPtr pWkbooks ;
	_WorkbookPtr pWkbook ;
	_WorksheetPtr pWksheet ;
	int nNumWkbooks ;
	int nCurrWkbookNum = 1 ;
	int nCurrWksheetNum = 1 ;
	
	pWkbooks = pApplication->Workbooks ;
	nNumWkbooks = pWkbooks->Count ;
	pWkbook = pWkbooks->GetItem(1) ;
	_bstr_t sCurrWkbookName = pWkbook->Name ;
	_bstr_t sWkbookName =  _bstr_t( sWorkbookName ) ;
	_bstr_t sWksheetName = _bstr_t( sWorksheetName ) ;

	while( sCurrWkbookName != sWkbookName && nCurrWkbookNum < nNumWkbooks )
	{
		nCurrWkbookNum = nCurrWkbookNum + 1 ;
		pWkbook = pWkbooks->GetItem(nCurrWkbookNum) ; 
		sCurrWkbookName = pWkbook->Name ;
	}

	if( sCurrWkbookName == sWkbookName )
	{
		pWksheet = pWkbook->Worksheets->GetItem(1) ;
		_bstr_t sCurrWksheetName = pWksheet->Name ;
		int nNumWksheets = pWkbook->Worksheets->Count ;
			
		while( sCurrWksheetName != sWksheetName && nCurrWksheetNum < nNumWksheets )
		{
			nCurrWksheetNum = nCurrWksheetNum + 1 ;
			pWksheet = pWkbook->Worksheets->GetItem(nCurrWksheetNum) ;
			sCurrWksheetName = pWksheet->Name ;
		}

		if ( sCurrWksheetName == sWksheetName )
		{
			pWkbooks->Release() ;
			return pWksheet ;
		}
	}
	pWkbooks->Release() ;
	return NULL ;
}


//////////////////////////////////////////////////////////////
//  fnGetWksheetNumPtr
_WorksheetPtr fnGetWksheetNumPtr
( LPSTR szWorkbookName, int nWorksheetNum ) 
{
	WorkbooksPtr pWkbooks ;
	_WorkbookPtr pWkbook ;
	_WorksheetPtr pWksheet ;
	int nNumWkbooks ;
	int nCurrWkbookNum = 1 ;
	int nCurrWksheetNum = 1 ;
	
	pWkbooks = pApplication->Workbooks ;
	nNumWkbooks = pWkbooks->Count ;
	pWkbook = pWkbooks->GetItem(1) ;
	_bstr_t sCurrWkbookName = pWkbook->Name ;
	_bstr_t sWkbookName =  _bstr_t( szWorkbookName ) ;

	while( sCurrWkbookName != sWkbookName && nCurrWkbookNum < nNumWkbooks )
	{
		nCurrWkbookNum = nCurrWkbookNum + 1 ;
		pWkbook = pWkbooks->GetItem(nCurrWkbookNum) ;
		sCurrWkbookName = pWkbook->Name ;
	}

	if( sCurrWkbookName == sWkbookName )
	{
		pWksheet = pWkbook->Worksheets->GetItem(1) ;
		int nNumWksheets = pWkbook->Worksheets->Count ;
		
		while( (int) pWksheet->Index != nWorksheetNum && nCurrWksheetNum < nNumWksheets )
		{
			nCurrWksheetNum = nCurrWksheetNum + 1 ;
			pWksheet = pWkbook->Worksheets->GetItem( nCurrWksheetNum ) ;
		}
		
		if ( (int) pWksheet->Index == nWorksheetNum )
		{
			pWkbooks->Release() ;
			return pWksheet ;
		}
	}
	pWkbooks->Release() ;
	return 0 ;
}


//////////////////////////////////////////////////////////////
//  fnWorkbookIsOpen
BOOL fnWorkbookIsOpen
( LPSTR sWorkbookName )
{
	WorkbooksPtr pWkbooks ;
	_WorkbookPtr pWkbook ;
	int nNumWkbooks ;
	int nCurrWkbookNum = 1 ;
	
	pWkbooks = pApplication->Workbooks ;
	nNumWkbooks = pWkbooks->Count ;
	
	if( nNumWkbooks )
	{
		pWkbook = pWkbooks->GetItem(1) ;
		_bstr_t sCurrWkbookName = pWkbook->Name ;
		_bstr_t sWkbookName =  _bstr_t( sWorkbookName ) ;

		while( sCurrWkbookName != sWkbookName && nCurrWkbookNum < nNumWkbooks )
		{
			nCurrWkbookNum = nCurrWkbookNum + 1 ;
			pWkbook = pWkbooks->GetItem( nCurrWkbookNum ) ;
			sCurrWkbookName = pWkbook->Name ;
		}

		if( sCurrWkbookName == sWkbookName ) {
			pWkbooks->Release() ;
			return TRUE ;
		}
	}
	pWkbooks->Release() ;
	return FALSE ;
}


//////////////////////////////////////////////////////////////
//  fnGenRuntimeError - Generates run-time errors in
//  TradeStation
void fnGenRuntimeError
( IEasyLanguageObject * pEL, int iErrorNum )
{
	TSRuntimeErrorItem tsItem;
	int m_HistErr ;
	tsItem.sCompany = _bstr_t("TradeStation Securities, Inc.").copy();
	tsItem.sErrorLocation = _bstr_t("ELExcel3.dll Example Function Library").copy();
	tsItem.sErrorCategory = _bstr_t("Error").copy();
	tsItem.sLongString = NULL;
	tsItem.nParameters = 0;
		
    switch ( iErrorNum )
	{
		case NOT_RUNNING_AND_ERROR_STARTING:
		{
			// Unable to locate a running instance of Excel and
			// unable to start a new instance.
			tsItem.sShortString = _bstr_t("Error - No running instance of"
				" Excel could be found.  Unable to launch a new instance.  "
				"DLL function ELXL_StartExcel.").copy();
			tsItem.sSourceString = _bstr_t("The Excel application must"
				" be installed on the computer.  However, rather than calling"
				" ELXL_StartExcel, try opening desired Excel workspace first,"
				" then call ELXL_Init.").copy();
			tsItem.nErrorCode = NOT_RUNNING_AND_ERROR_STARTING ;
			break ;
		}

		case ERROR_SETTING_VISIBILITY:
		{
			// Unable to make Excel instance visible
			tsItem.sShortString = _bstr_t("Error - Unable to set Excel"
				" visibility. DLL function ELXL_SetVisible.").copy();
			tsItem.sSourceString = _bstr_t("Try starting Excel and opening"
				" desired workspace first, then call ELXL_Init.").copy();
			tsItem.nErrorCode = ERROR_SETTING_VISIBILITY ;
			break ;
		}

		case INVALID_WORKSHEET_PTR:
		{
			// Error obtaining valid worksheet pointer.
			// Worksheet pointer is NULL.
			tsItem.sShortString = _bstr_t("Error obtaining worksheet pointer.").copy();
			tsItem.sSourceString = _bstr_t("Worksheet name parameter should not "
				" contain extended path information.  For example, specify worksheet "
				" name as ""MyWorksheet.xls"", rather than as "
				"""C:\\Directory\\MyWorksheet.xls"".").copy();
			tsItem.nErrorCode = INVALID_WORKSHEET_PTR ;
			break ;
		}
		
		case ERROR_SETTING_USING_WKSHEET:
		{
			tsItem.sShortString = _bstr_t("Error setting worksheet to use.").copy();
			tsItem.sSourceString = _bstr_t("Ensure that worksheet is open in Excel first."
				"  Also, Worksheet name parameter should not contain extended path"
				" information.  For example, specify worksheet name as ""MyWorksheet.xls"","
				" rather than as ""C:\\Directory\\MyWorksheet.xls"".").copy();
			tsItem.nErrorCode = ERROR_SETTING_USING_WKSHEET ;
			break ;
		}

		case ERROR_CLEARING_CELL_CONTENTS:
		{
			tsItem.sShortString = _bstr_t("Error clearing cell contents.").copy();
			tsItem.sSourceString = _bstr_t("Ensure that worksheet is open in Excel.").copy();
			tsItem.nErrorCode = ERROR_CLEARING_CELL_CONTENTS ;
			break ;
		}

		case ERROR_CLEARING_CELL_FORMAT:
		{
			tsItem.sShortString = _bstr_t("Error clearing cell format.").copy();
			tsItem.sSourceString = _bstr_t("Ensure that worksheet is open in Excel.").copy();
			tsItem.nErrorCode = ERROR_CLEARING_CELL_FORMAT ;
			break ;
		}

		case ERROR_CLEARING_SHEET_CONTENTS:
		{
			tsItem.sShortString = _bstr_t("Error clearing sheet contents.").copy();
			tsItem.sSourceString = _bstr_t("Ensure that worksheet is open in Excel.").copy();
			tsItem.nErrorCode = ERROR_CLEARING_SHEET_CONTENTS ;
			break ;
		}

		case ERROR_CLEARING_SHEET_FORMAT:
		{
			tsItem.sShortString = _bstr_t("Error clearing sheet format.").copy();
			tsItem.sSourceString = _bstr_t("Ensure that worksheet is open in Excel.").copy();
			tsItem.nErrorCode = ERROR_CLEARING_SHEET_FORMAT ;
			break ;
		}

		case ERROR_SETTING_CELL_TO_STRING:
		{
			// Error placing a string value in a cell
			tsItem.sShortString = _bstr_t("Error - Unable to put string value"
				" into cell.  DLL function ELXL_SetCellValueString.").copy();
			tsItem.sSourceString = _bstr_t("Check to ensure that value to be "
				" placed in cell is a string value, not a number or Boolean value."
				"  Also, verify that ELXL_UsingWorksheet was called prior to"
				" attempting to place value in cell.").copy();
			tsItem.nErrorCode = ERROR_SETTING_CELL_TO_STRING ;
			break ;
		}

		case ERROR_SETTING_CELL_TO_NUMBER:
		{
			// Error placing a numeric value in a cell
			tsItem.sShortString = _bstr_t("Error - Unable to put numeric value"
				" into cell.  DLL function ELXL_SetCellValueNumber.").copy();
			tsItem.sSourceString = _bstr_t("Check to ensure that value to be "
				" placed in cell is a numeric value, not a string or Boolean value."
				"  Also, verify that ELXL_UsingWorksheet was called prior to"
				" attempting to place value in cell").copy();
			tsItem.nErrorCode = ERROR_SETTING_CELL_TO_NUMBER ;
			break ;
		}

		case ERROR_SETTING_CELL_TO_BOOLEAN:
		{
			// Error placing a Boolean value in a cell
			tsItem.sShortString = _bstr_t("Error - Unable to put Boolean value"
				" into cell.  DLL function ELXL_SetCellValueBoolean.").copy();
			tsItem.sSourceString = _bstr_t("Check to ensure that value to be "
				" placed in cell is a Boolean value, not a string or numeric value."
				"  Also, verify that ELXL_UsingWorksheet was called prior to"
				" attempting to place value in cell").copy();
			tsItem.nErrorCode = ERROR_SETTING_CELL_TO_BOOLEAN ;
			break ;
		}

		case ERROR_GETTING_STRING_FROM_CELL:
		{
			// Error obtaining string value from a cell.
			tsItem.sShortString = _bstr_t("Error obtaining string value from"
				" cell.  DLL function ELXL_GetCellValueString.").copy();
			tsItem.sSourceString = _bstr_t("Check to ensure that value to be "
				" retrieved from cell is a string value, not a Boolean or numeric value."
				"  Also, verify that ELXL_UsingWorksheet was called prior to"
				" attempting to retrieve value from cell").copy();
			tsItem.nErrorCode = ERROR_GETTING_STRING_FROM_CELL ;
			break ;
		}
		
		case ERROR_GETTING_NUMBER_FROM_CELL:
		{
			// Error obtaining numeric value from a cell.
			tsItem.sShortString = _bstr_t("Error obtaining numeric value from"
				" cell.  DLL function ELXL_GetCellValueNumber.").copy();
			tsItem.sSourceString = _bstr_t("Check to ensure that value to be "
				" retrieved from cell is a numeric value, not a Boolean or string value."
				"  Also, verify that ELXL_UsingWorksheet was called prior to"
				" attempting to retrieve value from cell").copy();
			tsItem.nErrorCode = ERROR_GETTING_NUMBER_FROM_CELL ;
			break ;
		}

		case ERROR_GETTING_BOOLEAN_FROM_CELL:
		{
			// Error obtaining Boolean value from a cell.
			tsItem.sShortString = _bstr_t("Error obtaining Boolean value from"
				" cell.  DLL function ELXL_GetCellValueBoolean.").copy();
			tsItem.sSourceString = _bstr_t("Check to ensure that value to be "
				" retrieved from cell is a Boolean value, not a numeric or string value."
				"  Also, verify that ELXL_UsingWorksheet was called prior to"
				" attempting to retrieve value from cell").copy();
			tsItem.nErrorCode = ERROR_GETTING_BOOLEAN_FROM_CELL ;
			break ;
		}

		case ERROR_PERFORMING_CLEANUP:
		{
			// Error performing "OnDestroy" clean-up operations
			tsItem.sShortString = _bstr_t("Error obtaining Boolean value from"
				" cell.").copy();
			tsItem.sSourceString = _bstr_t("Check to ensure that value to be "
				" retrieved from cell is a Boolean value, not a numeric or string value."
				"  Also, verify that ELXL_UsingWorksheet was called prior to"
				" attempting to retrieve value from cell").copy();
			tsItem.nErrorCode = ERROR_PERFORMING_CLEANUP ;
			break ;
		}

		default:
		{
			// Generate a run-time error of undefined type or origin
			tsItem.sShortString = _bstr_t("Undefined error in "
				"ELExcel3.dll.").copy();
			tsItem.sSourceString = _bstr_t("Error origin unknown.").copy();
			tsItem.nErrorCode = UNDEFINED_ERR ;
		}
	}
	m_HistErr = pEL->Errors->RegisterError( &tsItem ) ;
	pEL->Errors->RaiseRuntimeError( m_HistErr ) ;
}
